#!/usr/bin/perl

# Grabber for overwriting data by Loxone data

# Copyright 2016-2018 Michael Schlenstedt, michael@loxberry.de
#                       Christian Fenzl, christian@loxberry.de
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

use strict;
use warnings;

##########################################################################
# procedure - modified for FOSHKplugin by Oliver Engel
# - needs adjustment in fetch.pl and a new grabber_local.pl in bin
# - needs LOCALGRABBER=1 and LOCAL\URL in conf
# 1. copy current.dat to current.dat.tmp
# 2. read current.dat.tmp into array A
# 3. read http-current.dat into array B
# 4. exchange all filled fields from 2 (B) into 1 (A)
# 5. write array A to current.dat.tmp on disk
# 6. copy current.dat.tmp --> current.dat
##########################################################################

##########################################################################
# Modules
##########################################################################

use LoxBerry::System;
use LoxBerry::Log;
use LoxBerry::IO;
use LWP::UserAgent;
#use JSON qw( decode_json );
use File::Copy;
use Getopt::Long;
use Time::Piece;
use Encode;

##########################################################################
# Read Settings
##########################################################################

# Version of this script
my $version = "0.0.0.1";

my $currentnametmp = "$lbplogdir/current.dat.tmp";
my $currentname    = "$lbplogdir/current.dat";
my $pcfg           = new Config::Simple("$lbpconfigdir/weather4lox.cfg");
my $locqueryurl    = $pcfg->param("LOCAL.URL");

if (not defined $locqueryurl) {
        LOGCRIT "You have to specify the URL of local weather station in LOCAL\\URL in config-file";
        exit 2;
}

# Read language phrases
my %L = LoxBerry::System::readlanguage("language.ini");

# Create a logging object
my $log = LoxBerry::Log->new (
        package => 'weather4lox',
        name => 'grabber_local',
        logdir => "$lbplogdir",
        #filename => "$lbplogdir/weather4lox.log",
        #append => 1,
);

# Commandline options
my $verbose = '';

GetOptions ('verbose' => \$verbose,
            'quiet'   => sub { $verbose = 0 });

# Due to a bug in the Logging routine, set the loglevel fix to 3
#$log->loglevel(3);
if ($verbose) {
        $log->stdout(1);
        $log->loglevel(7);
}

LOGSTART "Weather4Lox GRABBER_LOCAL process started";
LOGDEB "This is $0 Version $version";


LOGINF "Fetching weather data from local weather station via FOSHKplugin";

my $response_success;
my $ua = new LWP::UserAgent;
my $resp = $ua->get($locqueryurl);

# Check status of request
my $urlstatus = $resp->status_line;
my $urlstatuscode = substr($urlstatus,0,3);

LOGDEB "Status: $urlstatus";

if ($urlstatuscode ne "200") {
  LOGCRIT "Failed to fetch data for $locqueryurl\. Status Code: $urlstatuscode";
  exit 2;
} else {
  LOGOK "Data fetched successfully for $locqueryurl";
}

my $content = $resp->decoded_content();
$content = encode('utf8', decode('latin1',$content));

LOGDEB "Copying current.dat to current.dat.tmp";
copy($currentname, $currentnametmp);

LOGINF "Reading current.dat.tmp";

my $datafile_str = LoxBerry::System::read_file($currentnametmp);
chomp($datafile_str);

LOGDEB "Old line: $datafile_str";

my @oldvalues = split /\|/, $datafile_str,-1;
my @newvalues = split /\|/, $content,-1;

for (my $i=0;$i<=$#oldvalues;$i++) {
  next if($oldvalues[$i] eq $newvalues[$i] or $newvalues[$i] eq "" or $newvalues[$i] eq "-9999");
  LOGDEB "  value in field $i ($oldvalues[$i]) is set to $newvalues[$i]";
  $oldvalues[$i] = $newvalues[$i];
  #$oldvalues[$i] =~ s/^([-\d\.]+).*/$1/g;
}

# Joining line
my $newline = join('|', @oldvalues);

LOGDEB "New line: $newline";

# Write patched file
eval {
        open(my $fh, ">$currentnametmp");
        #binmode $fh, ':encoding(UTF-8)';
        print $fh $newline;
        close $fh;
}
or do {
    LOGCRIT "Could not write $currentnametmp: $@";
        exit 2;
};

# Test file
my $currentsize = -s ($currentnametmp);
if ($currentsize > 100) {
        move($currentnametmp, $currentname);
} else {
        LOGCRIT "File size below 100 bytes - no new file created: $currentnametmp";
        exit 2;
}

# Give OK status to client.
LOGOK "Current Data saved successfully.";

# Exit
exit;

END
{
        LOGEND;
}
